@@ -1,29 +1,24 @@
 use crate::acc_tracker::AccTracker;
-use crate::{max, min, FuturesType, Margin, Order, OrderType, Position, Side};
+use crate::{max, min, FuturesTypes, Margin, Order, OrderType, Position, Side};
 
 #[derive(Debug, Clone)]
 /// The users account
 pub struct Account {
-    futures_type: FuturesType,
+    futures_type: FuturesTypes,
     margin: Margin,
     position: Position,
     acc_tracker: AccTracker,
     active_limit_orders: Vec<Order>,
-    active_stop_orders: Vec<Order>,
     executed_orders: Vec<Order>,
     // used for calculating hedged order size for order margin calculation
     pub(crate) open_limit_buy_size: f64,
     pub(crate) open_limit_sell_size: f64,
-    pub(crate) open_stop_buy_size: f64,
-    pub(crate) open_stop_sell_size: f64,
     pub(crate) min_limit_buy_price: f64,
     pub(crate) max_limit_sell_price: f64,
-    pub(crate) max_stop_buy_price: f64,
-    pub(crate) min_stop_sell_price: f64,
 }
 
 impl Account {
-    pub fn new(leverage: f64, starting_balance: f64, futures_type: FuturesType) -> Self {
+    pub fn new(leverage: f64, starting_balance: f64, futures_type: FuturesTypes) -> Self {
         let position = Position::new(leverage);
         let margin = Margin::new_init(starting_balance);
         let acc_tracker = AccTracker::new(starting_balance);
@@ -33,16 +28,11 @@ impl Account {
             position,
             acc_tracker,
             active_limit_orders: vec![],
-            active_stop_orders: vec![],
             executed_orders: vec![],
             open_limit_buy_size: 0.0,
             open_limit_sell_size: 0.0,
-            open_stop_buy_size: 0.0,
-            open_stop_sell_size: 0.0,
             min_limit_buy_price: 0.0,
             max_limit_sell_price: 0.0,
-            max_stop_buy_price: 0.0,
-            min_stop_sell_price: 0.0,
         }
     }
 
@@ -85,8 +75,7 @@ impl Account {
 
     /// Return recently executed orders
     /// and clear them afterwards
-    #[deprecated]
-    pub fn executed_orders(&mut self) -> Vec<Order> {
+    pub(crate) fn executed_orders(&mut self) -> Vec<Order> {
         let exec_orders = self.executed_orders.clone();
         self.executed_orders.clear();
 
@@ -99,12 +88,6 @@ impl Account {
         &self.active_limit_orders
     }
 
-    /// Return the currently active stop orders
-    #[inline(always)]
-    pub fn active_stop_orders(&self) -> &Vec<Order> {
-        &self.active_stop_orders
-    }
-
     /// Return a reference to acc_tracker struct
     #[inline(always)]
     pub fn acc_tracker(&self) -> &AccTracker {
@@ -115,75 +98,43 @@ impl Account {
     /// returns Some order if successful with given order_id
     pub fn cancel_order(&mut self, order_id: u64) -> Option<Order> {
         for (i, o) in self.active_limit_orders.iter().enumerate() {
-            if o.id == order_id {
+            if o.id() == order_id {
                 let old_order = self.active_limit_orders.remove(i);
-                match old_order.side {
-                    Side::Buy => self.open_limit_buy_size -= old_order.size,
-                    Side::Sell => self.open_limit_sell_size -= old_order.size,
-                }
-                return Some(old_order);
-            }
-        }
-        for (i, o) in self.active_stop_orders.iter().enumerate() {
-            if o.id == order_id {
-                let old_order = self.active_stop_orders.remove(i);
-                match old_order.side {
-                    Side::Buy => self.open_stop_buy_size -= old_order.size,
-                    Side::Sell => self.open_stop_sell_size -= old_order.size,
+                match old_order.side() {
+                    Side::Buy => self.open_limit_buy_size -= old_order.size(),
+                    Side::Sell => self.open_limit_sell_size -= old_order.size(),
                 }
                 return Some(old_order);
             }
         }
+
         // re compute min and max prices for open orders
         self.min_limit_buy_price = 0.0;
         self.max_limit_sell_price = 0.0;
         for o in self.active_limit_orders.iter() {
-            match o.side {
+            let limit_price = o.limit_price().unwrap();
+            match o.side() {
                 Side::Buy => {
                     if self.min_limit_buy_price == 0.0 {
-                        self.min_limit_buy_price = o.limit_price;
+                        self.min_limit_buy_price = limit_price;
                         continue;
                     }
-                    if o.limit_price < self.min_limit_buy_price {
-                        self.min_limit_buy_price = o.limit_price;
+                    if limit_price < self.min_limit_buy_price {
+                        self.min_limit_buy_price = limit_price;
                     }
                 }
                 Side::Sell => {
                     if self.max_limit_sell_price == 0.0 {
-                        self.max_limit_sell_price = o.limit_price;
+                        self.max_limit_sell_price = limit_price;
                         continue;
                     }
-                    if o.limit_price > self.max_limit_sell_price {
-                        self.max_limit_sell_price = o.limit_price;
+                    if limit_price > self.max_limit_sell_price {
+                        self.max_limit_sell_price = limit_price;
                     }
                 }
             }
         }
 
-        self.max_stop_buy_price = 0.0;
-        self.min_stop_sell_price = 0.0;
-        for o in self.active_stop_orders.iter() {
-            match o.side {
-                Side::Buy => {
-                    if self.max_stop_buy_price == 0.0 {
-                        self.max_stop_buy_price = o.trigger_price;
-                        continue;
-                    }
-                    if o.trigger_price > self.max_stop_buy_price {
-                        self.max_stop_buy_price = o.trigger_price;
-                    }
-                }
-                Side::Sell => {
-                    if self.min_stop_sell_price == 0.0 {
-                        self.min_stop_sell_price = o.trigger_price;
-                        continue;
-                    }
-                    if o.trigger_price < self.min_stop_sell_price {
-                        self.min_stop_sell_price = o.trigger_price;
-                    }
-                }
-            }
-        }
         None
     }
 
@@ -192,44 +143,39 @@ impl Account {
         self.margin.set_order_margin(0.0);
         self.open_limit_buy_size = 0.0;
         self.open_limit_sell_size = 0.0;
-        self.open_stop_buy_size = 0.0;
-        self.open_stop_sell_size = 0.0;
         self.min_limit_buy_price = 0.0;
         self.max_limit_sell_price = 0.0;
-        self.max_stop_buy_price = 0.0;
-        self.min_stop_sell_price = 0.0;
         self.active_limit_orders.clear();
-        self.active_stop_orders.clear();
     }
 
     /// append order to active orders and update internal state accordingly
     pub(crate) fn append_order(&mut self, order: Order) {
-        match order.order_type {
+        match order.order_type() {
             OrderType::Limit => self.append_limit_order(order),
-            OrderType::StopMarket => self.append_stop_order(order),
             OrderType::Market => {}
         }
     }
 
     /// Append a new limit order as active order
     fn append_limit_order(&mut self, order: Order) {
-        match order.side {
+        let limit_price = order.limit_price().unwrap();
+        match order.side() {
             Side::Buy => {
-                self.open_limit_buy_size += order.size;
+                self.open_limit_buy_size += order.size();
                 if self.min_limit_buy_price == 0.0 {
-                    self.min_limit_buy_price = order.limit_price;
+                    self.min_limit_buy_price = limit_price;
                 }
-                if order.limit_price < self.min_limit_buy_price {
-                    self.min_limit_buy_price = order.limit_price;
+                if limit_price < self.min_limit_buy_price {
+                    self.min_limit_buy_price = limit_price;
                 }
             }
             Side::Sell => {
-                self.open_limit_sell_size += order.size;
+                self.open_limit_sell_size += order.size();
                 if self.max_limit_sell_price == 0.0 {
-                    self.max_limit_sell_price = order.limit_price;
+                    self.max_limit_sell_price = limit_price;
                 }
-                if order.limit_price > self.max_limit_sell_price {
-                    self.max_limit_sell_price = order.limit_price;
+                if limit_price > self.max_limit_sell_price {
+                    self.max_limit_sell_price = limit_price;
                 }
             }
         }
@@ -240,34 +186,6 @@ impl Account {
         self.active_limit_orders.push(order);
     }
 
-    /// Append a new stop order as active order
-    fn append_stop_order(&mut self, order: Order) {
-        match order.side {
-            Side::Buy => {
-                self.open_stop_buy_size += order.size;
-                if self.max_stop_buy_price == 0.0 {
-                    self.max_stop_buy_price = order.trigger_price;
-                }
-                if order.trigger_price > self.max_stop_buy_price {
-                    self.max_stop_buy_price = order.trigger_price;
-                }
-            }
-            Side::Sell => {
-                self.open_stop_sell_size += order.size;
-                if self.min_stop_sell_price == 0.0 {
-                    self.min_stop_sell_price = order.trigger_price;
-                }
-                if order.trigger_price < self.min_stop_sell_price {
-                    self.min_stop_sell_price = order.trigger_price;
-                }
-            }
-        }
-        // set order margin
-        self.margin.set_order_margin(self.order_margin());
-
-        self.active_stop_orders.push(order);
-    }
-
     /// Finalize an executed limit order
     pub(crate) fn finalize_limit_order(&mut self, order_idx: usize) {
         let mut exec_order = self.active_limit_orders.remove(order_idx);
@@ -275,9 +193,9 @@ impl Account {
         exec_order.mark_executed();
 
         // free order margin
-        match exec_order.side {
-            Side::Buy => self.open_limit_buy_size -= exec_order.size,
-            Side::Sell => self.open_limit_sell_size -= exec_order.size,
+        match exec_order.side() {
+            Side::Buy => self.open_limit_buy_size -= exec_order.size(),
+            Side::Sell => self.open_limit_sell_size -= exec_order.size(),
         }
         // re-calculate min and max price
         self.min_limit_buy_price = if self.active_limit_orders.is_empty() {
@@ -285,8 +203,8 @@ impl Account {
         } else {
             self.active_limit_orders
                 .iter()
-                .filter(|o| o.side == Side::Buy)
-                .map(|o| o.limit_price)
+                .filter(|o| o.side() == Side::Buy)
+                .map(|o| o.limit_price().unwrap())
                 .sum()
         };
         self.max_limit_sell_price = if self.active_limit_orders.is_empty() {
@@ -294,44 +212,8 @@ impl Account {
         } else {
             self.active_limit_orders
                 .iter()
-                .filter(|o| o.side == Side::Sell)
-                .map(|o| o.limit_price)
-                .sum()
-        };
-
-        self.margin.set_order_margin(self.order_margin());
-
-        self.executed_orders.push(exec_order);
-    }
-
-    /// Finalize an executed stop order
-    pub(crate) fn finalize_stop_order(&mut self, order_idx: usize) {
-        let mut exec_order = self.active_stop_orders.remove(order_idx);
-
-        exec_order.mark_executed();
-
-        // free order margin
-        match exec_order.side {
-            Side::Buy => self.open_stop_buy_size -= exec_order.size,
-            Side::Sell => self.open_stop_sell_size -= exec_order.size,
-        }
-        // re-calculate min and max price
-        self.min_stop_sell_price = if self.active_stop_orders.is_empty() {
-            0.0
-        } else {
-            self.active_stop_orders
-                .iter()
-                .filter(|o| o.side == Side::Sell)
-                .map(|o| o.trigger_price)
-                .sum()
-        };
-        self.max_stop_buy_price = if self.active_stop_orders.is_empty() {
-            0.0
-        } else {
-            self.active_stop_orders
-                .iter()
-                .filter(|o| o.side == Side::Buy)
-                .map(|o| o.trigger_price)
+                .filter(|o| o.side() == Side::Sell)
+                .map(|o| o.limit_price().unwrap())
                 .sum()
         };
 
@@ -349,12 +231,12 @@ impl Account {
     }
 
     /// Changes the position by a given delta while changing margin accordingly
-    pub(crate) fn change_position(&mut self, side: Side, size: f64, price: f64) {
+    pub(crate) fn change_position(&mut self, side: Side, size: f64, exec_price: f64) {
         trace!(
-            "account: change_position(side: {:?}, size: {}, price: {})",
+            "account: change_position(side: {:?}, size: {}, exec_price: {})",
             side,
             size,
-            price
+            exec_price
         );
         let pos_size_delta: f64 = match side {
             Side::Buy => size,
@@ -367,12 +249,12 @@ impl Account {
                     if size > self.position.size().abs() {
                         self.futures_type.pnl(
                             self.position.entry_price(),
-                            price,
+                            exec_price,
                             self.position.size(),
                         )
                     } else {
                         self.futures_type
-                            .pnl(self.position.entry_price(), price, -size)
+                            .pnl(self.position.entry_price(), exec_price, -size)
                     }
                 } else {
                     0.0
@@ -384,12 +266,12 @@ impl Account {
                     if size > self.position.size() {
                         self.futures_type.pnl(
                             self.position.entry_price(),
-                            price,
+                            exec_price,
                             self.position.size(),
                         )
                     } else {
                         self.futures_type
-                            .pnl(self.position.entry_price(), price, size)
+                            .pnl(self.position.entry_price(), exec_price, size)
                     }
                 } else {
                     0.0
@@ -403,13 +285,13 @@ impl Account {
 
         // change position
         self.position
-            .change_size(pos_size_delta, price, self.futures_type);
+            .change_size(pos_size_delta, exec_price, self.futures_type);
 
         // set position margin
         let mut pos_margin: f64 = self.position.size().abs() / self.position.leverage();
         match self.futures_type {
-            FuturesType::Linear => pos_margin *= self.position.entry_price(),
-            FuturesType::Inverse => pos_margin /= self.position.entry_price(),
+            FuturesTypes::Linear => pos_margin *= self.position.entry_price(),
+            FuturesTypes::Inverse => pos_margin /= self.position.entry_price(),
         };
         self.margin.set_position_margin(pos_margin);
 
@@ -421,12 +303,7 @@ impl Account {
     /// TODO: mark it work in all cases
     fn order_margin(&self) -> f64 {
         let ps: f64 = self.position.size();
-        let open_sizes: [f64; 4] = [
-            self.open_limit_buy_size,
-            self.open_limit_sell_size,
-            self.open_stop_buy_size,
-            self.open_stop_sell_size,
-        ];
+        let open_sizes: [f64; 2] = [self.open_limit_buy_size, self.open_limit_sell_size];
         let mut max_idx: usize = 0;
         let mut m: f64 = self.open_limit_buy_size;
 
@@ -441,8 +318,6 @@ impl Account {
         let (d, p) = match max_idx {
             0 => (1.0, self.min_limit_buy_price),
             1 => (-1.0, self.max_limit_sell_price),
-            2 => (1.0, self.max_stop_buy_price),
-            3 => (-1.0, self.min_stop_sell_price),
             _ => panic!("any other value should not be possible"),
         };
         if p == 0.0 {
@@ -456,12 +331,11 @@ impl Account {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::round;
-    use crate::FuturesType;
+    use crate::FuturesTypes;
 
     #[test]
     fn account_append_limit_order() {
-        let mut account = Account::new(1.0, 1.0, FuturesType::Inverse);
+        let mut account = Account::new(1.0, 1.0, FuturesTypes::Inverse);
 
         account.append_limit_order(Order::limit(Side::Buy, 100.0, 25.0).unwrap());
         assert_eq!(account.open_limit_buy_size, 25.0);
@@ -497,40 +371,9 @@ mod tests {
         // assert_eq!(account.margin().available_balance(), 0.75);
     }
 
-    #[test]
-    fn account_append_stop_order() {
-        let mut account = Account::new(1.0, 1.0, FuturesType::Inverse);
-
-        account.append_stop_order(Order::stop_market(Side::Buy, 100.0, 25.0).unwrap());
-        assert_eq!(account.open_stop_buy_size, 25.0);
-        assert_eq!(account.open_stop_sell_size, 0.0);
-        assert_eq!(account.max_stop_buy_price, 100.0);
-        assert_eq!(account.min_stop_sell_price, 0.0);
-
-        account.append_stop_order(Order::stop_market(Side::Sell, 100.0, 25.0).unwrap());
-        assert_eq!(account.open_stop_buy_size, 25.0);
-        assert_eq!(account.open_stop_sell_size, 25.0);
-        assert_eq!(account.max_stop_buy_price, 100.0);
-        assert_eq!(account.min_stop_sell_price, 100.0);
-
-        account.append_stop_order(Order::stop_market(Side::Buy, 110.0, 25.0).unwrap());
-        assert_eq!(account.open_stop_buy_size, 50.0);
-        assert_eq!(account.open_stop_sell_size, 25.0);
-        assert_eq!(account.max_stop_buy_price, 110.0);
-        assert_eq!(account.min_stop_sell_price, 100.0);
-
-        account.append_stop_order(Order::stop_market(Side::Sell, 90.0, 25.0).unwrap());
-        assert_eq!(account.open_stop_buy_size, 50.0);
-        assert_eq!(account.open_stop_sell_size, 50.0);
-        assert_eq!(account.max_stop_buy_price, 110.0);
-        assert_eq!(account.min_stop_sell_price, 90.0);
-
-        // TODO: test order_margin
-    }
-
     #[test]
     fn account_cancel_order() {
-        let mut account = Account::new(1.0, 1.0, FuturesType::Inverse);
+        let mut account = Account::new(1.0, 1.0, FuturesTypes::Inverse);
 
         let o = Order::limit(Side::Buy, 900.0, 450.0).unwrap();
         account.append_order(o);
@@ -546,7 +389,7 @@ mod tests {
 
     #[test]
     fn account_cancel_all_orders() {
-        let mut account = Account::new(1.0, 1.0, FuturesType::Inverse);
+        let mut account = Account::new(1.0, 1.0, FuturesTypes::Inverse);
 
         let o = Order::limit(Side::Buy, 900.0, 450.0).unwrap();
         account.append_order(o);
@@ -556,18 +399,8 @@ mod tests {
         assert_eq!(account.margin().order_margin(), 0.5);
         assert_eq!(account.margin().available_balance(), 0.5);
 
-        let o = Order::stop_market(Side::Buy, 1100.0, 450.0).unwrap();
-        account.append_order(o);
-        assert_eq!(account.active_limit_orders().len(), 1);
-        assert_eq!(account.active_stop_orders().len(), 1);
-        assert_eq!(account.margin().wallet_balance(), 1.0);
-        assert_eq!(account.margin().position_margin(), 0.0);
-        assert_eq!(round(account.margin().order_margin(), 1), 0.5);
-        assert_eq!(account.margin().available_balance(), 0.5);
-
         account.cancel_all_orders();
         assert_eq!(account.active_limit_orders().len(), 0);
-        assert_eq!(account.active_stop_orders().len(), 0);
         assert_eq!(account.margin().wallet_balance(), 1.0);
         assert_eq!(account.margin().position_margin(), 0.0);
         assert_eq!(account.margin().order_margin(), 0.0);
@@ -577,7 +410,7 @@ mod tests {
 
     #[test]
     fn account_order_margin() {
-        let mut account = Account::new(1.0, 1.0, FuturesType::Inverse);
+        let mut account = Account::new(1.0, 1.0, FuturesTypes::Inverse);
 
         account.append_order(Order::limit(Side::Buy, 100.0, 50.0).unwrap());
         assert_eq!(account.order_margin(), 0.5);
@@ -585,28 +418,16 @@ mod tests {
         account.append_order(Order::limit(Side::Sell, 100.0, 50.0).unwrap());
         assert_eq!(account.order_margin(), 0.5);
 
-        account.append_order(Order::stop_market(Side::Sell, 100.0, 50.0).unwrap());
-        assert_eq!(account.order_margin(), 0.5);
-
-        account.append_order(Order::stop_market(Side::Buy, 100.0, 50.0).unwrap());
-        assert_eq!(account.order_margin(), 0.5);
-
         account.append_order(Order::limit(Side::Buy, 100.0, 50.0).unwrap());
         assert_eq!(account.order_margin(), 1.0);
 
         account.append_order(Order::limit(Side::Sell, 100.0, 50.0).unwrap());
         assert_eq!(account.order_margin(), 1.0);
-
-        account.append_order(Order::stop_market(Side::Sell, 100.0, 50.0).unwrap());
-        assert_eq!(account.order_margin(), 1.0);
-
-        account.append_order(Order::stop_market(Side::Buy, 100.0, 50.0).unwrap());
-        assert_eq!(account.order_margin(), 1.0);
     }
 
     #[test]
     fn account_change_position_size_inverse_future() {
-        let mut acc = Account::new(1.0, 1.0, FuturesType::Inverse);
+        let mut acc = Account::new(1.0, 1.0, FuturesTypes::Inverse);
 
         acc.change_position(Side::Buy, 100.0, 200.0);
         assert_eq!(acc.margin().wallet_balance(), 1.0);
