@@ -1,7 +1,4 @@
-use crate::{Account, Config, FuturesType, Order, OrderError, OrderType, Side, Validator};
-
-const MAX_NUM_LIMIT_ORDERS: usize = 50;
-const MAX_NUM_STOP_ORDERS: usize = 50;
+use crate::{Account, Config, FuturesTypes, Order, OrderError, OrderType, Side, Validator};
 
 #[derive(Debug, Clone)]
 /// The main leveraged futures exchange for simulated trading
@@ -20,13 +17,16 @@ pub struct Exchange {
 impl Exchange {
     /// Create a new Exchange with the desired config and whether to use candles as infomation source
     pub fn new(config: Config) -> Exchange {
-        assert!(config.leverage > 0.0);
         let account = Account::new(
-            config.leverage,
-            config.starting_balance,
-            config.futures_type,
+            config.leverage(),
+            config.starting_balance(),
+            config.futures_type(),
+        );
+        let validator = Validator::new(
+            config.fee_maker(),
+            config.fee_taker(),
+            config.futures_type(),
         );
-        let validator = Validator::new(config.fee_maker, config.fee_taker, config.futures_type);
         Exchange {
             config,
             account,
@@ -66,10 +66,17 @@ impl Exchange {
 
     /// Return a mutable reference to Account
     #[inline(always)]
+    #[deprecated] // use set_account instead
     pub fn account_mut(&mut self) -> &mut Account {
         &mut self.account
     }
 
+    /// Set the account, use carefully
+    #[inline(always)]
+    pub fn set_account(&mut self, account: Account) {
+        self.account = account
+    }
+
     /// Update the exchange state with new information
     /// ### Parameters
     /// bid: bid price
@@ -80,6 +87,7 @@ impl Exchange {
     /// ### Returns
     /// executed orders
     /// true if position has been liquidated
+    #[must_use]
     pub fn update_state(
         &mut self,
         bid: f64,
@@ -114,6 +122,33 @@ impl Exchange {
         (self.account.executed_orders(), false)
     }
 
+    /// Submit a new order to the exchange.
+    /// Returns the order with timestamp and id filled in or OrderError
+    #[must_use]
+    pub fn submit_order(&mut self, mut order: Order) -> Result<Order, OrderError> {
+        self.validator.validate(&order, &self.account)?;
+
+        // assign unique order id
+        order.set_id(self.next_order_id);
+        self.next_order_id += 1;
+
+        order.set_timestamp(self.step as i64);
+
+        match order.order_type() {
+            OrderType::Market => {
+                // immediately execute market order
+                self.execute_market(order.side(), order.size());
+
+                Ok(order)
+            }
+            _ => {
+                self.account.append_order(order);
+
+                Ok(order)
+            }
+        }
+    }
+
     /// Check if a liquidation event should occur
     fn check_liquidation(&mut self) -> bool {
         // TODO: check_liquidation
@@ -134,10 +169,10 @@ impl Exchange {
             Side::Sell => self.bid,
         };
 
-        let mut fee = self.config.fee_taker * amount;
-        match self.config.futures_type {
-            FuturesType::Linear => fee *= price,
-            FuturesType::Inverse => fee /= price,
+        let mut fee = self.config.fee_taker() * amount;
+        match self.config.futures_type() {
+            FuturesTypes::Linear => fee *= price,
+            FuturesTypes::Inverse => fee /= price,
         }
         self.account.deduce_fees(fee);
         self.account.change_position(side, amount, price);
@@ -148,10 +183,10 @@ impl Exchange {
         // TODO: log_limit_order_fill
         //self.account.acc_tracker_mut().log_limit_order_fill();
 
-        let mut fee = self.config.fee_maker * amount;
-        match self.config.futures_type {
-            FuturesType::Linear => fee *= price,
-            FuturesType::Inverse => fee /= price,
+        let mut fee = self.config.fee_maker() * amount;
+        match self.config.futures_type() {
+            FuturesTypes::Linear => fee *= price,
+            FuturesTypes::Inverse => fee /= price,
         }
         self.account.deduce_fees(fee);
         self.account.change_position(side, amount, price);
@@ -172,57 +207,30 @@ impl Exchange {
     /// method is called after new external data has been consumed
     fn check_orders(&mut self) {
         for i in 0..self.account.active_limit_orders().len() {
-            match self.account.active_limit_orders()[i].order_type {
+            match self.account.active_limit_orders()[i].order_type() {
                 OrderType::Limit => self.handle_limit_order(i),
                 _ => panic!("there should only be limit orders in active_limit_orders"),
             }
         }
-        for i in 0..self.account.active_stop_orders().len() {
-            match self.account.active_stop_orders()[i].order_type {
-                OrderType::StopMarket => self.handle_stop_market_order(i),
-                _ => panic!("there should only be stop market orders in active_stop_orders"),
-            }
-        }
-    }
-
-    /// Handle stop market order trigger and execution
-    fn handle_stop_market_order(&mut self, order_idx: usize) {
-        // check if stop order has been triggered
-        match self.account().active_stop_orders()[order_idx].side {
-            Side::Buy => {
-                if self.account().active_stop_orders()[order_idx].trigger_price > self.high {
-                    return;
-                }
-            }
-            Side::Sell => {
-                if self.account().active_stop_orders()[order_idx].trigger_price < self.low {
-                    return;
-                }
-            }
-        }
-        self.execute_market(
-            self.account().active_stop_orders()[order_idx].side,
-            self.account().active_stop_orders()[order_idx].size,
-        );
-        self.account.finalize_stop_order(order_idx);
     }
 
     /// Handle limit order trigger and execution
     fn handle_limit_order(&mut self, order_idx: usize) {
         let o: Order = self.account.active_limit_orders()[order_idx];
-        match o.side {
+        let limit_price = o.limit_price().unwrap();
+        match o.side() {
             Side::Buy => {
                 // use candle information to specify execution
-                if self.low <= o.limit_price {
-                    self.execute_limit(o.side, o.limit_price, o.size);
+                if self.low <= limit_price {
+                    self.execute_limit(o.side(), limit_price, o.size());
                 } else {
                     return;
                 }
             }
             Side::Sell => {
                 // use candle information to specify execution
-                if self.high >= o.limit_price {
-                    self.execute_limit(o.side, o.limit_price, o.size);
+                if self.high >= limit_price {
+                    self.execute_limit(o.side(), limit_price, o.size());
                 } else {
                     return;
                 }
@@ -230,43 +238,4 @@ impl Exchange {
         }
         self.account.finalize_limit_order(order_idx);
     }
-
-    /// Submit a new order to the exchange.
-    /// Returns the order with timestamp and id filled in or OrderError
-    pub fn submit_order(&mut self, mut order: Order) -> Result<Order, OrderError> {
-        match order.order_type {
-            OrderType::StopMarket => {
-                if self.account().active_limit_orders().len() >= MAX_NUM_LIMIT_ORDERS {
-                    return Err(OrderError::MaxActiveOrders);
-                }
-            }
-            _ => {
-                if self.account().active_stop_orders().len() >= MAX_NUM_STOP_ORDERS {
-                    return Err(OrderError::MaxActiveOrders);
-                }
-            }
-        }
-
-        self.validator.validate(&order, &self.account)?;
-
-        // assign unique order id
-        order.id = self.next_order_id;
-        self.next_order_id += 1;
-
-        order.timestamp = self.step;
-
-        match order.order_type {
-            OrderType::Market => {
-                // immediately execute market order
-                self.execute_market(order.side, order.size);
-
-                Ok(order)
-            }
-            _ => {
-                self.account.append_order(order);
-
-                Ok(order)
-            }
-        }
-    }
 }
