@@ -4,15 +4,9 @@ use lfest::*;
 
 #[test]
 fn submit_order_limit() {
-    let config = Config {
-        fee_maker: -0.00025,
-        fee_taker: 0.00075,
-        starting_balance: 1.0,
-        leverage: 1.0,
-        futures_type: FuturesType::Inverse,
-    };
+    let config = Config::new(0.0002, 0.0006, 1.0, 1.0, FuturesTypes::Inverse).unwrap();
     let mut exchange = Exchange::new(config);
-    exchange.update_state(1000.0, 1000.0, 0, 1000.0, 1000.0);
+    let _ = exchange.update_state(1000.0, 1000.0, 0, 1000.0, 1000.0);
 
     // submit working market order
     let o = Order::market(Side::Buy, 500.0).unwrap();
@@ -33,40 +27,13 @@ fn test_handle_limit_order() {
     // TODO:
 }
 
-#[test]
-fn handle_stop_market_order_w_trade() {
-    let config = Config {
-        fee_maker: -0.00025,
-        fee_taker: 0.00075,
-        starting_balance: 1.0,
-        leverage: 1.0,
-        futures_type: FuturesType::Inverse,
-    };
-    let mut exchange = Exchange::new(config);
-    exchange.update_state(1000.0, 1000.0, 0, 1000.0, 1000.0);
-
-    let o = Order::stop_market(Side::Buy, 1010.0, 100.0).unwrap();
-    exchange.submit_order(o).unwrap();
-    assert_eq!(exchange.account().active_stop_orders().len(), 1);
-
-    exchange.update_state(1010.0, 1010.0, 1, 1010.0, 1010.0);
-
-    assert_eq!(exchange.account().position().size(), 100.0);
-    assert_eq!(exchange.account().position().entry_price(), 1010.0);
-}
-
 #[test]
 fn inv_long_market_win_full() {
-    let config = Config {
-        fee_maker: -0.00025,
-        fee_taker: 0.00075,
-        starting_balance: 1.0,
-        leverage: 1.0,
-        futures_type: FuturesType::Inverse,
-    };
-    let fee_taker = config.fee_taker;
+    let config = Config::new(0.0002, 0.0006, 1.0, 1.0, FuturesTypes::Inverse).unwrap();
+
+    let fee_taker = config.fee_taker();
     let mut exchange = Exchange::new(config);
-    exchange.update_state(1000.0, 1000.0, 0, 1000.0, 1000.0);
+    let _ = exchange.update_state(1000.0, 1000.0, 0, 1000.0, 1000.0);
 
     let value = exchange.account().margin().available_balance() * 0.8;
     let size = exchange.ask() * value;
@@ -74,7 +41,7 @@ fn inv_long_market_win_full() {
     let order_err = exchange.submit_order(o);
     assert!(order_err.is_ok());
 
-    exchange.update_state(1000.0, 1000.0, 1, 1000.0, 1000.0);
+    let _ = exchange.update_state(1000.0, 1000.0, 1, 1000.0, 1000.0);
 
     let fee_base = size * fee_taker;
     let fee_asset1 = fee_base / exchange.bid();
@@ -92,7 +59,7 @@ fn inv_long_market_win_full() {
         round(0.2 - fee_asset1, 4)
     );
 
-    exchange.update_state(2000.0, 2000.0, 1, 2000.0, 2000.0);
+    let _ = exchange.update_state(2000.0, 2000.0, 1, 2000.0, 2000.0);
     assert_eq!(exchange.account().position().unrealized_pnl(), 0.4);
 
     let size = 800.0;
@@ -120,21 +87,16 @@ fn inv_long_market_win_full() {
 fn inv_long_market_loss_full() {
     if let Err(_) = pretty_env_logger::try_init() {}
 
-    let config = Config {
-        fee_maker: 0.0,
-        fee_taker: 0.0,
-        starting_balance: 1.0,
-        leverage: 1.0,
-        futures_type: FuturesType::Inverse,
-    };
-    let fee_taker = config.fee_taker;
+    let config = Config::new(0.0002, 0.0006, 1.0, 1.0, FuturesTypes::Inverse).unwrap();
+
+    let fee_taker = config.fee_taker();
     let mut exchange = Exchange::new(config);
-    exchange.update_state(1000.0, 1000.0, 0, 1000.0, 1000.0);
+    let _ = exchange.update_state(1000.0, 1000.0, 0, 1000.0, 1000.0);
 
     let o = Order::market(Side::Buy, 800.0).unwrap();
     let order_err = exchange.submit_order(o);
     assert!(order_err.is_ok());
-    exchange.update_state(1000.0, 1000.0, 1, 1000.0, 1000.0);
+    let _ = exchange.update_state(1000.0, 1000.0, 1, 1000.0, 1000.0);
 
     assert_eq!(exchange.account().position().size(), 800.0);
     assert_eq!(exchange.account().position().entry_price(), 1000.0);
@@ -147,12 +109,12 @@ fn inv_long_market_loss_full() {
     assert_eq!(exchange.account().margin().order_margin(), 0.0);
     assert_eq!(exchange.account().margin().position_margin(), 0.8);
 
-    exchange.update_state(800.0, 800.0, 2, 800.0, 800.0);
+    let _ = exchange.update_state(800.0, 800.0, 2, 800.0, 800.0);
     assert_eq!(exchange.account().position().unrealized_pnl(), -0.2);
 
     let o = Order::market(Side::Sell, 800.0).unwrap();
     exchange.submit_order(o).unwrap();
-    exchange.update_state(800.0, 800.0, 3, 800.0, 800.0);
+    let _ = exchange.update_state(800.0, 800.0, 3, 800.0, 800.0);
 
     let fee_base0 = fee_taker * 800.0;
     let fee_asset0 = fee_base0 / 1000.0;
@@ -177,31 +139,26 @@ fn inv_long_market_loss_full() {
 
 #[test]
 fn inv_short_market_win_full() {
-    let config = Config {
-        fee_maker: -0.00025,
-        fee_taker: 0.00075,
-        starting_balance: 1.0,
-        leverage: 1.0,
-        futures_type: FuturesType::Inverse,
-    };
-    let fee_taker = config.fee_taker;
+    let config = Config::new(0.0002, 0.0006, 1.0, 1.0, FuturesTypes::Inverse).unwrap();
+
+    let fee_taker = config.fee_taker();
     let mut exchange = Exchange::new(config);
-    exchange.update_state(1000.0, 1000.0, 0, 1000.0, 1000.0);
+    let _ = exchange.update_state(1000.0, 1000.0, 0, 1000.0, 1000.0);
 
     let o = Order::market(Side::Sell, 800.0).unwrap();
     let order_err = exchange.submit_order(o);
     assert!(order_err.is_ok());
-    exchange.update_state(1000.0, 1000.0, 1, 1000.0, 1000.0);
+    let _ = exchange.update_state(1000.0, 1000.0, 1, 1000.0, 1000.0);
 
     assert_eq!(exchange.account().position().size(), -800.0);
 
-    exchange.update_state(800.0, 800.0, 1, 800.0, 800.0);
+    let _ = exchange.update_state(800.0, 800.0, 1, 800.0, 800.0);
     assert_eq!(exchange.account().position().unrealized_pnl(), 0.2);
 
     let o = Order::market(Side::Buy, 800.0).unwrap();
     let order_err = exchange.submit_order(o);
     assert!(order_err.is_ok());
-    exchange.update_state(800.0, 800.0, 2, 800.0, 800.0);
+    let _ = exchange.update_state(800.0, 800.0, 2, 800.0, 800.0);
 
     let fee_base0 = fee_taker * 800.0;
     let fee_asset0 = fee_base0 / 1000.0;
@@ -226,16 +183,11 @@ fn inv_short_market_win_full() {
 
 #[test]
 fn inv_short_market_loss_full() {
-    let config = Config {
-        fee_maker: -0.00025,
-        fee_taker: 0.00075,
-        starting_balance: 1.0,
-        leverage: 1.0,
-        futures_type: FuturesType::Inverse,
-    };
-    let fee_taker = config.fee_taker;
+    let config = Config::new(0.0002, 0.0006, 1.0, 1.0, FuturesTypes::Inverse).unwrap();
+
+    let fee_taker = config.fee_taker();
     let mut exchange = Exchange::new(config);
-    exchange.update_state(1000.0, 1000.0, 0, 1000.0, 1000.0);
+    let _ = exchange.update_state(1000.0, 1000.0, 0, 1000.0, 1000.0);
 
     let value = exchange.account().margin().available_balance() * 0.4;
     let size = exchange.ask() * value;
@@ -243,7 +195,7 @@ fn inv_short_market_loss_full() {
     let order_err = exchange.submit_order(o);
     assert!(order_err.is_ok());
 
-    exchange.update_state(1000.0, 1000.0, 1, 1000.0, 1000.0);
+    let _ = exchange.update_state(1000.0, 1000.0, 1, 1000.0, 1000.0);
 
     let fee_base1 = size * fee_taker;
     let fee_asset1 = fee_base1 / exchange.bid();
@@ -261,7 +213,7 @@ fn inv_short_market_loss_full() {
         0.6 - fee_asset1
     );
 
-    exchange.update_state(2000.0, 2000.0, 2, 2000.0, 2000.0);
+    let _ = exchange.update_state(2000.0, 2000.0, 2, 2000.0, 2000.0);
 
     let size = 400.0;
     let fee_base2 = size * fee_taker;
@@ -273,7 +225,7 @@ fn inv_short_market_loss_full() {
     let order_err = exchange.submit_order(o);
     assert!(order_err.is_ok());
 
-    exchange.update_state(2000.0, 2000.0, 3, 2000.0, 2000.0);
+    let _ = exchange.update_state(2000.0, 2000.0, 3, 2000.0, 2000.0);
 
     assert_eq!(exchange.account().position().size(), 0.0);
     assert_eq!(exchange.account().position().unrealized_pnl(), 0.0);
@@ -290,16 +242,11 @@ fn inv_short_market_loss_full() {
 
 #[test]
 fn inv_long_market_win_partial() {
-    let config = Config {
-        fee_maker: -0.00025,
-        fee_taker: 0.00075,
-        starting_balance: 1.0,
-        leverage: 1.0,
-        futures_type: FuturesType::Inverse,
-    };
-    let fee_taker = config.fee_taker;
+    let config = Config::new(0.0002, 0.0006, 1.0, 1.0, FuturesTypes::Inverse).unwrap();
+
+    let fee_taker = config.fee_taker();
     let mut exchange = Exchange::new(config);
-    exchange.update_state(1000.0, 1000.0, 0, 1000.0, 1000.0);
+    let _ = exchange.update_state(1000.0, 1000.0, 0, 1000.0, 1000.0);
 
     let value = exchange.account().margin().available_balance() * 0.8;
     let size = exchange.ask() * value;
@@ -307,7 +254,7 @@ fn inv_long_market_win_partial() {
     let order_err = exchange.submit_order(o);
     assert!(order_err.is_ok());
 
-    exchange.update_state(1000.0, 1000.0, 1, 1000.0, 1000.0);
+    let _ = exchange.update_state(1000.0, 1000.0, 1, 1000.0, 1000.0);
 
     let fee_base = size * fee_taker;
     let fee_asset1 = fee_base / exchange.bid();
@@ -325,7 +272,7 @@ fn inv_long_market_win_partial() {
         round(0.2 - fee_asset1, 4)
     );
 
-    exchange.update_state(2000.0, 2000.0, 1, 2000.0, 2000.0);
+    let _ = exchange.update_state(2000.0, 2000.0, 1, 2000.0, 2000.0);
 
     let size = 400.0;
     let fee_base2 = size * fee_taker;
@@ -337,7 +284,7 @@ fn inv_long_market_win_partial() {
     let order_err = exchange.submit_order(o);
     assert!(order_err.is_ok());
 
-    exchange.update_state(2000.0, 2000.0, 2, 2000.0, 2000.0);
+    let _ = exchange.update_state(2000.0, 2000.0, 2, 2000.0, 2000.0);
 
     assert_eq!(exchange.account().position().size(), 400.0);
     assert_eq!(exchange.account().position().entry_price(), 1000.0);
@@ -355,31 +302,26 @@ fn inv_long_market_win_partial() {
 
 #[test]
 fn inv_long_market_loss_partial() {
-    let config = Config {
-        fee_maker: -0.00025,
-        fee_taker: 0.00075,
-        starting_balance: 1.0,
-        leverage: 1.0,
-        futures_type: FuturesType::Inverse,
-    };
-    let fee_taker = config.fee_taker;
+    let config = Config::new(0.0002, 0.0006, 1.0, 1.0, FuturesTypes::Inverse).unwrap();
+
+    let fee_taker = config.fee_taker();
     let mut exchange = Exchange::new(config);
-    exchange.update_state(1000.0, 1000.0, 0, 1000.0, 1000.0);
+    let _ = exchange.update_state(1000.0, 1000.0, 0, 1000.0, 1000.0);
 
     let o = Order::market(Side::Buy, 800.0).unwrap();
     let order_err = exchange.submit_order(o);
     assert!(order_err.is_ok());
-    exchange.update_state(1000.0, 1000.0, 1, 1000.0, 1000.0);
+    let _ = exchange.update_state(1000.0, 1000.0, 1, 1000.0, 1000.0);
 
     assert_eq!(exchange.account().position().size(), 800.0);
 
-    exchange.update_state(800.0, 800.0, 1, 800.0, 800.0);
+    let _ = exchange.update_state(800.0, 800.0, 1, 800.0, 800.0);
     assert_eq!(exchange.account().position().unrealized_pnl(), -0.2);
 
     let o = Order::market(Side::Sell, 400.0).unwrap();
     let order_err = exchange.submit_order(o);
     assert!(order_err.is_ok());
-    exchange.update_state(800.0, 800.0, 1, 800.0, 800.0);
+    let _ = exchange.update_state(800.0, 800.0, 1, 800.0, 800.0);
 
     let fee_base0 = fee_taker * 800.0;
     let fee_asset0 = fee_base0 / 1000.0;
@@ -404,21 +346,16 @@ fn inv_long_market_loss_partial() {
 
 #[test]
 fn inv_short_market_win_partial() {
-    let config = Config {
-        fee_maker: 0.0,
-        fee_taker: 0.00075,
-        starting_balance: 1.0,
-        leverage: 1.0,
-        futures_type: FuturesType::Inverse,
-    };
-    let fee_taker = config.fee_taker;
+    let config = Config::new(0.0002, 0.0006, 1.0, 1.0, FuturesTypes::Inverse).unwrap();
+
+    let fee_taker = config.fee_taker();
     let mut exchange = Exchange::new(config);
-    exchange.update_state(1000.0, 1000.0, 0, 1000.0, 1000.0);
+    let _ = exchange.update_state(1000.0, 1000.0, 0, 1000.0, 1000.0);
 
     let o = Order::market(Side::Sell, 800.0).unwrap();
     let order_err = exchange.submit_order(o);
     assert!(order_err.is_ok());
-    exchange.update_state(1000.0, 1000.0, 1, 1000.0, 1000.0);
+    let _ = exchange.update_state(1000.0, 1000.0, 1, 1000.0, 1000.0);
 
     assert_eq!(exchange.account().position().size(), -800.0);
     assert_eq!(exchange.account().position().entry_price(), 1000.0);
@@ -431,14 +368,14 @@ fn inv_short_market_win_partial() {
     assert_eq!(exchange.account().margin().order_margin(), 0.0);
     assert_eq!(exchange.account().margin().position_margin(), 0.8);
 
-    exchange.update_state(800.0, 800.0, 2, 800.0, 800.0);
+    let _ = exchange.update_state(800.0, 800.0, 2, 800.0, 800.0);
 
     assert_eq!(exchange.account().position().unrealized_pnl(), 0.2);
 
     let o = Order::market(Side::Buy, 400.0).unwrap();
     let order_err = exchange.submit_order(o);
     assert!(order_err.is_ok());
-    exchange.update_state(800.0, 800.0, 3, 800.0, 800.0);
+    let _ = exchange.update_state(800.0, 800.0, 3, 800.0, 800.0);
 
     let fee_base0 = fee_taker * 800.0;
     let fee_asset0 = fee_base0 / 1000.0;
@@ -463,16 +400,11 @@ fn inv_short_market_win_partial() {
 
 #[test]
 fn inv_short_market_loss_partial() {
-    let config = Config {
-        fee_maker: -0.00025,
-        fee_taker: 0.00075,
-        starting_balance: 1.0,
-        leverage: 1.0,
-        futures_type: FuturesType::Inverse,
-    };
-    let fee_taker = config.fee_taker;
+    let config = Config::new(0.0002, 0.0006, 1.0, 1.0, FuturesTypes::Inverse).unwrap();
+
+    let fee_taker = config.fee_taker();
     let mut exchange = Exchange::new(config);
-    exchange.update_state(1000.0, 1000.0, 0, 1000.0, 1000.0);
+    let _ = exchange.update_state(1000.0, 1000.0, 0, 1000.0, 1000.0);
 
     let value = exchange.account().margin().available_balance() * 0.8;
     let size = exchange.ask() * value;
@@ -480,7 +412,7 @@ fn inv_short_market_loss_partial() {
     let order_err = exchange.submit_order(o);
     assert!(order_err.is_ok());
 
-    exchange.update_state(1000.0, 1000.0, 1, 1000.0, 1000.0);
+    let _ = exchange.update_state(1000.0, 1000.0, 1, 1000.0, 1000.0);
 
     let fee_base1 = size * fee_taker;
     let fee_asset1 = fee_base1 / exchange.bid();
@@ -497,7 +429,7 @@ fn inv_short_market_loss_partial() {
         round(exchange.account().margin().available_balance(), 4),
         round(0.2 - fee_asset1, 4)
     );
-    exchange.update_state(2000.0, 2000.0, 1, 2000.0, 2000.0);
+    let _ = exchange.update_state(2000.0, 2000.0, 1, 2000.0, 2000.0);
 
     let size = 400.0;
     let fee_base2 = size * fee_taker;
@@ -524,23 +456,18 @@ fn inv_short_market_loss_partial() {
 
 #[test]
 fn inv_test_market_roundtrip() {
-    let config = Config {
-        fee_maker: -0.00025,
-        fee_taker: 0.00075,
-        starting_balance: 1.0,
-        leverage: 1.0,
-        futures_type: FuturesType::Inverse,
-    };
-    let fee_taker = config.fee_taker;
+    let config = Config::new(0.0002, 0.0006, 1.0, 1.0, FuturesTypes::Inverse).unwrap();
+
+    let fee_taker = config.fee_taker();
     let mut exchange = Exchange::new(config);
-    exchange.update_state(1000.0, 1000.0, 0, 1000.0, 1000.0);
+    let _ = exchange.update_state(1000.0, 1000.0, 0, 1000.0, 1000.0);
 
     let value = exchange.account().margin().available_balance() * 0.9;
     let size = exchange.ask() * value;
     let buy_order = Order::market(Side::Buy, size).unwrap();
     let order_err = exchange.submit_order(buy_order);
     assert!(order_err.is_ok());
-    exchange.update_state(1000.0, 1000.0, 1, 1000.0, 1000.0);
+    let _ = exchange.update_state(1000.0, 1000.0, 1, 1000.0, 1000.0);
 
     let sell_order = Order::market(Side::Sell, size).unwrap();
 
@@ -550,7 +477,7 @@ fn inv_test_market_roundtrip() {
     let fee_base = size * fee_taker;
     let fee_asset = fee_base / exchange.ask();
 
-    exchange.update_state(1000.0, 1000.0, 2, 1000.0, 1000.0);
+    let _ = exchange.update_state(1000.0, 1000.0, 2, 1000.0, 1000.0);
 
     assert_eq!(exchange.account().position().size(), 0.0);
     assert_eq!(exchange.account().position().entry_price(), 1000.0);
@@ -569,7 +496,7 @@ fn inv_test_market_roundtrip() {
     let buy_order = Order::market(Side::Buy, size).unwrap();
     let order_err = exchange.submit_order(buy_order);
     assert!(order_err.is_ok());
-    exchange.update_state(1000.0, 1000.0, 3, 1000.0, 1000.0);
+    let _ = exchange.update_state(1000.0, 1000.0, 3, 1000.0, 1000.0);
 
     let size = 950.0;
     let sell_order = Order::market(Side::Sell, size).unwrap();
@@ -577,7 +504,7 @@ fn inv_test_market_roundtrip() {
     let order_err = exchange.submit_order(sell_order);
     assert!(order_err.is_ok());
 
-    exchange.update_state(1000.0, 1000.0, 4, 1000.0, 1000.0);
+    let _ = exchange.update_state(1000.0, 1000.0, 4, 1000.0, 1000.0);
 
     assert_eq!(exchange.account().position().size(), -50.0);
     assert_eq!(exchange.account().position().entry_price(), 1000.0);
@@ -599,15 +526,10 @@ fn test_liquidate() {
 
 #[test]
 fn inv_execute_limit() {
-    let config = Config {
-        fee_maker: 0.0,
-        fee_taker: 0.001,
-        starting_balance: 1.0,
-        leverage: 1.0,
-        futures_type: FuturesType::Inverse,
-    };
+    let config = Config::new(0.0002, 0.0006, 1.0, 1.0, FuturesTypes::Inverse).unwrap();
+
     let mut exchange = Exchange::new(config.clone());
-    exchange.update_state(1000.0, 1000.0, 0, 1000.0, 1000.0);
+    let _ = exchange.update_state(1000.0, 1000.0, 0, 1000.0, 1000.0);
 
     let o: Order = Order::limit(Side::Buy, 900.0, 450.0).unwrap();
     exchange.submit_order(o).unwrap();
@@ -631,7 +553,7 @@ fn inv_execute_limit() {
     assert!(order_err.is_ok());
     assert_eq!(exchange.account().active_limit_orders().len(), 1);
 
-    exchange.update_state(1200.0, 1200.0, 1, 1200.0, 1200.0);
+    let _ = exchange.update_state(1200.0, 1200.0, 1, 1200.0, 1200.0);
 
     assert_eq!(exchange.account().active_limit_orders().len(), 0);
     assert_eq!(exchange.account().position().size(), 0.0);
@@ -643,7 +565,7 @@ fn inv_execute_limit() {
     exchange.submit_order(o).unwrap();
     assert_eq!(exchange.account().active_limit_orders().len(), 1);
 
-    exchange.update_state(1200.0, 1201.0, 2, 1201.0, 1200.0);
+    let _ = exchange.update_state(1200.0, 1201.0, 2, 1201.0, 1200.0);
     assert_eq!(exchange.account().position().size(), -600.0);
     assert_eq!(round(exchange.account().margin().position_margin(), 1), 0.5);
     assert_eq!(round(exchange.account().margin().wallet_balance(), 2), 1.05);
